package edu.harvard.mgh.purcell.gplink.forms;


import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;


import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.harvard.mgh.purcell.gplink.base_form.*;
import edu.harvard.mgh.purcell.gplink.mainGUI.*;


/**
 * Form for the clustering command --cluster
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class ClusteringForm extends Forms {

	//There are lots of options for this filter
	Forms.myCheckBox ppcButton, ccButton, kButton, ibmButton, 
		userButton,	 neighButton, pmergeButton,
		matchButton, matchTButton,
		qmatchButton, mcButton, mccButton;
	
	Forms.myJTextField minText, readGText, ppcText, pibsGText, mcText, 
		mcc1Text, mcc2Text, kText, ibmText, 
		matchText, matchTText, qmatchText, qtText;
	
	Forms.pickAFileButton readGBrowse, matchBrowse, matchTBrowse, 
		qmatchBrowse, qmatchQTBrowse;
	
	
	static Double PIBS_G_DEFAULT = new Double(500);
	static Double PCC_DEFAULT = new Double(0);
		
	
		
	/**
	 * create a show a new Clustering Form
	 * @param mf MainFrame this form is attached to
	 */
	public ClusteringForm(MainFrame mf) {
		super(mf, "Clustering");
		//doens't make any sence to have the filter or threshold
		//...optionsn turned on for this form
		this.buttons.disableFT();
	}

	/**
	 * create the body of the form
	 *
	 */
	@Override
	protected void createBody() {
		//make the layout easy
		body.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		JPanel readGPanel = new JPanel();
		readGPanel.setLayout(new BoxLayout(readGPanel, BoxLayout.LINE_AXIS));
		readGPanel.setLayout(new GridBagLayout());
		GridBagConstraints c1 = new GridBagConstraints();
		c1.fill = GridBagConstraints.HORIZONTAL;
		readGPanel.add(new JLabel("IBS distance file (--read-genome)"), c1);
		c1.weightx = 1;
		c1.gridx = 1;
		readGPanel.add(readGText, c1);
		c1.weightx = 0;
		c1.gridx = 2;
		readGPanel.add(readGBrowse, c1);
		body.add(readGPanel, c);

		
		//add the optional constraints
		JPanel constraints = new JPanel();
		constraints.setBorder(
				new TitledBorder("Optional clustering constraints"));
		constraints.setLayout(new GridBagLayout());
		c1 = new GridBagConstraints();
		c1.fill = GridBagConstraints.HORIZONTAL;
		c1.weightx = 1;
		JPanel ppcPanel = new JPanel();
		ppcPanel.setLayout(new GridBagLayout());
		GridBagConstraints c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		ppcPanel.add(ppcButton, c2);
		c2.gridx = 1;
		ppcPanel.add(ppcText, c2);
		c2.gridx = 2;
		ppcPanel.add(new JLabel("--ppc-gap "), c2);
		c2.gridx = 3;
		ppcPanel.add(pibsGText, c2);
		c2.gridx = 4;
		c2.weightx = 1;
		//This feels like a hack but I can't think of the pretty way to do this
		ppcPanel.add(new JLabel(), c2); 
		constraints.add(ppcPanel, c1);
		JPanel ibmPanel = new JPanel();
		ibmPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		ibmPanel.add(ibmButton, c2);
		c2.gridx = 1;
		ibmPanel.add(ibmText, c2);
		c2.gridx = 2;
		c2.weightx = 1;
		ibmPanel.add(new JLabel(), c2);
		c1.gridy = 1;
		constraints.add(ibmPanel, c1);
		
		c1.gridy = 2;
		constraints.add(ccButton, c1);
		JPanel mcPanel = new JPanel();
		mcPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		mcPanel.add(mcButton, c2);
		c2.gridx = 1;
		mcPanel.add(mcText, c2);
		c2.gridx = 2;
		c2.weightx = 1;
		mcPanel.add(new JLabel(), c2);
		c1.gridy = 3;
		constraints.add(mcPanel, c1);
		
		JPanel mccPanel = new JPanel();
		mccPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		mccPanel.add(mccButton, c2);
		c2.gridx = 1;
		mccPanel.add(mcc1Text, c2);
		c2.gridx = 2;
		mccPanel.add(mcc2Text, c2);
		c2.gridx = 3;
		c2.weightx = 1;
		mccPanel.add(new JLabel(), c2);
		c1.gridy = 4;
		constraints.add(mccPanel, c1);
		JPanel kPanel = new JPanel();
		kPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		kPanel.add(kButton, c2);
		c2.gridx = 1;
		kPanel.add(kText, c2);
		c2.gridx = 2;
		c2.weightx = 1;
		kPanel.add(new JLabel(), c2);
		c1.gridy = 5;
		constraints.add(kPanel, c1);

		
		JPanel matchPanel = new JPanel();
		matchPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		matchPanel.add(matchButton, c2);
		c2.gridx = 1;
		c2.weightx = 1;
		matchPanel.add(matchText, c2);
		c2.gridx = 2;
		c2.weightx = 0;
		matchPanel.add(matchBrowse, c2);
		c1.gridy = 6;
		constraints.add(matchPanel, c1);
		JPanel matchTPanel = new JPanel();
		matchTPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		matchTPanel.add(matchTButton, c2);
		c2.gridx = 1;
		c2.weightx = 1;
		matchTPanel.add(matchTText, c2);
		c2.gridx = 2;
		c2.weightx = 0;
		matchTPanel.add(matchTBrowse, c2);
		c1.gridy = 7;
		constraints.add(matchTPanel, c1);
		JPanel qmatchQTPanel = new JPanel();
		qmatchQTPanel.setLayout(new GridBagLayout());
		c2 = new GridBagConstraints();
		c2.fill = GridBagConstraints.HORIZONTAL;
		qmatchQTPanel.add(qmatchButton, c2);
		c2.gridwidth = 1;
		c2.gridx = 1;
		c2.weightx = 1;
		qmatchQTPanel.add(qmatchText, c2);
		c2.weightx = 0;
		c2.gridx = 2;
		qmatchQTPanel.add(qmatchBrowse, c2);
		c2.gridy = 1;
		c2.gridx = 0;
		c2.fill = GridBagConstraints.NONE;
		c2.anchor = GridBagConstraints.EAST;
		JLabel qtlabel = new JLabel("Thresholds (--qt) ");
		qmatchQTPanel.add(qtlabel, c2);
		c2.weightx = 1;
		c2.gridx = 1;
		c2.fill = GridBagConstraints.HORIZONTAL;
		qmatchQTPanel.add(qtText, c2);
		c2.weightx = 0;
		c2.gridx = 2;
		qmatchQTPanel.add(qmatchQTBrowse, c2);
		c1.gridy = 8;
		constraints.add(qmatchQTPanel, c1);
		
		c.gridy = 1;
		body.add(constraints, c);
	}

	/**
	 * initialize the variables
	 *
	 */
	@Override
	protected void initalize() {
		body = new JPanel();
		
		readGText = new myJTextField(10);
		readGText.setEditable(true);
		ppcText = new myJTextField(PCC_DEFAULT.toString(), 4);
		pibsGText = new myJTextField(PIBS_G_DEFAULT.toString(), 4);
		mcText = new myJTextField(4);
		mcc1Text = new myJTextField(4);
		mcc2Text = new myJTextField(4);
		kText = new myJTextField(4);
		ibmText = new myJTextField(4); 
		matchText = new myJTextField(10);
		matchTText = new myJTextField(10);
		qmatchText = new myJTextField(10);
		qtText = new myJTextField(10);
		
		readGBrowse = new pickAFileButton(".genome", "GENOME", readGText);
		matchBrowse = new pickAFileButton(".match", "MATCH", matchText);
		matchBrowse.setEnabled(false);
		matchTBrowse = new pickAFileButton(".bt", "MATCH-TYPE", matchTText);
		matchTBrowse.setEnabled(false);
		qmatchBrowse = new pickAFileButton(".match", "MATCH", qmatchText);
		qmatchBrowse.setEnabled(false);
		qmatchQTBrowse = new pickAFileButton(".qt", "QT", qtText);
		qmatchQTBrowse.setEnabled(false);
		
		myJTextField [] temp = {ppcText, pibsGText};
		ppcButton = new myCheckBox("Pairwise population concordance threshold (--pcc)", temp);
		ibmButton = new myCheckBox("Identity by missingness (--ibm)"
				, ibmText);
		ccButton = new myCheckBox("Phenotype constraint (-cc)");
		kButton = new myCheckBox("Number of clusters (--K)"
				, kText);
		matchButton = new myCheckBox("External categorical matching " +
				"(--match)", matchText, matchBrowse); 
		matchTButton = new myCheckBox("Positive/negative matches" +
				"(--match-type)", matchTText, matchTBrowse); 
		myJTextField [] temp3 = {qmatchText, qtText};
		pickAFileButton [] temp4 = {qmatchBrowse, qmatchQTBrowse};
		qmatchButton = new myCheckBox("External quantitative matching " +
				"(--qmatch)", temp3, temp4);
		mcButton = new myCheckBox("Maximum cluster size (--mc)", mcText); 
		myJTextField [] temp2 = {mcc1Text, mcc2Text};
		mccButton = new myCheckBox("Maximum number of cases/controls per " +
				"cluster (--mcc)", temp2);
		
	}

	/**
	 * 
	 * @return if the body is a valid command
	 */
	@Override
	protected boolean isBodyValid() {
		
		boolean ans = readGText.getText().length() > 0;
		
		if(ppcButton.isSelected())
			ans = ans && !ppcText.getText().equals("")
				&& !pibsGText.getText().equals("");
		if(ibmButton.isSelected())
			ans = ans && !ibmText.getText().equals("");
		if(mcButton.isSelected())
			ans = ans && !mcText.getText().equals("");
		if(mccButton.isSelected())
			ans = ans && !mcc1Text.getText().equals("")
				&& !mcc2Text.getText().equals("");
		if(kButton.isSelected())
			ans = ans && !kText.getText().equals("");
		if(matchButton.isSelected())
			ans = ans && !matchText.getText().equals("");
		if(matchTButton.isSelected())
			ans = ans && !matchTText.getText().equals("");
		if(qmatchButton.isSelected())
			ans = ans && !qmatchText.getText().equals("")
				&& !qtText.getText().equals("");
		
//		if(ans == false)
//			information.setText("There are either conflicting flags\n" +
//					"set or a selected flag does not have a required parameter.");
		
		return (ans);
	}

	/**
	 * process the body of the form to generate the plink command
	 * @return the command generated by this form
	 */
	@Override
	protected String processBody() {
		String ans = "--read-genome " 
			+ GeneralUtil.quoteFilename(readGText.getText())
			+ " --cluster";
		
		if(ppcButton.isSelected()){
			ans += " --ppc " + ppcText.getText();
			ans += " --ppc-gap " + pibsGText.getText();
		}
		if(ccButton.isSelected()){
			ans +=" --cc";
		}
		if(mcButton.isSelected()){
			ans += " --mc " + mcText.getText();
		}
		if(mccButton.isSelected()){
			ans += " --mcc " + mcc1Text.getText() + " " + mcc2Text.getText();
		}
		if(kButton.isSelected()){
			ans += " --K " + kText.getText();
		}
		if(ibmButton.isSelected()){
			ans += " --ibm " + ibmText.getText();
		} 
		if(matchButton.isSelected()){
			ans += " --match " + GeneralUtil.quoteFilename(matchText.getText());
		}
		if(matchTButton.isSelected()){
			ans += " --match-type " + GeneralUtil.quoteFilename(matchTText.getText());
		}
		if(qmatchButton.isSelected()){
			ans += " --qmatch " + GeneralUtil.quoteFilename(qmatchText.getText())
				+ " --qt " + GeneralUtil.quoteFilename(qtText.getText());
		}
		
		return ans;
	}

}
