package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
//import java.io.FileInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

//import ch.ethz.ssh2.SCPClient;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.data.*;

/**
 * Set plink wide parameters in this dialog: path to plink, any constent prefix
 * to plink commands, flag the system as a window's system
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class ConfigDialog extends JDialog {

	//link to the project being affected
	private Project data;

	//field that holds the plink path
	private JTextField plinkP;

	//field that holds the prefix that will be added to all
	//...commands generated by in this project
	private JTextField prefix;
	
	//holds the haploview path
	private JTextField haploviewPath;

	//holds the appending for haploview command
	private JTextField haploviewAppend;
	
	//name of the user specified editor
	private JTextField userEditorName;
	
	
	//link to the main frame
	private MainFrame frame;
	
	//let the user pick if thye want to use the default editor or
	//...specify their own
	private JLabel defaultEditor;
	private JCheckBox userEditor;
	private Properties config;
	
	@SuppressWarnings("static-access")
	private void writeGobalConfig(){
		/*
		 * Grab the configurations in the dialog
		 */
		if(frame.menuStuff.browseOnly.isSelected())
			config.setProperty(data.BROWSE_KEY, data.TRUE);
		else
			config.setProperty(data.BROWSE_KEY, data.FALSE);
		
		config.setProperty(data.PLINK_PATH_KEY, plinkP.getText());
		config.setProperty(data.PLINK_PREFIX, prefix.getText());
		config.setProperty(data.HAPLOVIEW_PATH_KEY, haploviewPath.getText());
		config.setProperty(data.HAPLOVIEW_APPEND, haploviewAppend.getText());
		
		if(userEditor.isSelected())
			config.setProperty(data.USER_EDITOR_KEY, userEditorName.getText());
		else
			config.setProperty(data.USER_EDITOR_KEY, "");
		
		
		//load old local config
		Properties localConfig = new Properties();
		File localConfigFile = new File(System.getProperty("user.dir")
				+ File.separator + data.GLOBAL_CONFIG_FILE);
		
		try{
			localConfig.load(new FileInputStream(localConfigFile));
		}catch(Exception e1){
			localConfigFile = new File(System.getProperty("user.home")
					+ File.separator + data.GLOBAL_CONFIG_FILE);
			try{
				localConfig.load(new FileInputStream(localConfigFile));
			}catch(Exception e2){
				//its ok to fail quietly
			}
		}
		
		//load remote config file
		Properties remoteConfig = new Properties();
		String remoteConfigFile = "~/" + data.GLOBAL_CONFIG_FILE;
		if(data.isRemote(false) == true){
			DownloadFile remoteConfigDownload = new DownloadFile(
					remoteConfigFile, data.getRoot().getAbsolutePath(), 
					data.getConn(), true);
			remoteConfigDownload.start();
			try {
				remoteConfigDownload.join();
			} catch (InterruptedException e1) {
				new ErrorDialog(frame, "Download of remote config file interupted.\n" +
				"writeGobalConfig@ConfigDialog.java");
			}
			try {/*
				SCPClient scopy = new SCPClient(data.getConn());
				scopy.get(remoteConfigFile, data.getRoot().getAbsolutePath());
				*/remoteConfig.load(new FileInputStream(new File(data.getRoot(), data.GLOBAL_CONFIG_FILE)));
			} catch (IOException e) {
				//it will all get taken care off later we just don't
				//set the remote properties "right" nothing breaks
			}
		}
		
		//change the values to reflect what's in the dialog
		localConfig.setProperty(data.BROWSE_KEY, config.getProperty(data.BROWSE_KEY));
		localConfig.setProperty(data.HAPLOVIEW_PATH_KEY, config.getProperty(data.HAPLOVIEW_PATH_KEY));
		localConfig.setProperty(data.HAPLOVIEW_APPEND, config.getProperty(data.HAPLOVIEW_APPEND));
		localConfig.setProperty(data.USER_EDITOR_KEY, config.getProperty(data.USER_EDITOR_KEY));
		
		if(data.isRemote(false) == true){
			remoteConfig.setProperty(data.PLINK_PATH_KEY, config.getProperty(data.PLINK_PATH_KEY));
			remoteConfig.setProperty(data.PLINK_PREFIX, config.getProperty(data.PLINK_PREFIX));
		} else {
			localConfig.setProperty(data.PLINK_PATH_KEY, config.getProperty(data.PLINK_PATH_KEY));
			localConfig.setProperty(data.PLINK_PREFIX, config.getProperty(data.PLINK_PREFIX));
		
		}
		//save the remote config
		if(data.isRemote(true)
				&& !data.getBrowse()){
			File tempConfigFile = new File(data.getRoot(), data.GLOBAL_CONFIG_FILE );
			try {
				remoteConfig.store(new FileOutputStream(tempConfigFile), 
					"gPLINK global configuration");
			} catch (FileNotFoundException e) {
				new ErrorDialog(frame, "Can not save the configureation files \n" +
						"writeGobalConfig@ConfigDialog.java");
			} catch (IOException e) {
				new ErrorDialog(frame, "Can not save the configureation files \n" +
				"writeGobalConfig@ConfigDialog.java");
			}
			new UploadFile(data.frame, (tempConfigFile.getAbsolutePath()), 
						"~/", data.getConn()).start();
			/*
			try{
				SCPClient scopy = new SCPClient(data.getConn());
				//System.out.println("ConfigDialog: scopy.put("+tempConfigFile.getAbsolutePath()+", "+
				//		GeneralUtil.makeSpacesHard("~/")+", " + data.SCP_MODE + ")");
				scopy.put((tempConfigFile.getAbsolutePath()), 
						"~/", data.SCP_MODE);	
			} catch (IOException e){
				new ErrorDialog(frame, "scopy.put(" + tempConfigFile.getAbsolutePath() + ", " +
						GeneralUtil.makeSpacesHard("~/") + ", " + data.SCP_MODE+ ") \n" +
						"Can not copy the remote configuratoin file: \n" +
						"writeGobalConfig@ConfigDialog.java");
				e.printStackTrace();
			}
			*/
			
			//tempConfigFile.delete();
		}
		
		//save the local config
		try{
			localConfig.store(new FileOutputStream(localConfigFile), 
					"gPLINK global configuration");
		}catch(IOException e){
				new ErrorDialog(frame, "Error trying to store the "
						+ "local config file.\n writeGobalConfig@ConfigDailog.java");
		}
	}

	/**
	 * Sets all the correct parameters from the dialog
	 */
	ActionListener setConfigValues = new ActionListener() {
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e) {
			//set the path for plink
			data.setPlinkPath(plinkP.getText());
			//set the prefix for plink
			data.setPlinkPrefix(prefix.getText());
			//tell GPLINK where haploview is
			data.setHaploviewPath(haploviewPath.getText());
			data.setHaploviewAppend(haploviewAppend.getText());
			//check to see if the default editor is selected
			if(userEditor.isSelected())
				//set it based on what operating system you have
				data.setEditor(userEditorName.getText());
			else
				//set it based on what is in the text field
				data.setEditor(null);
			
			writeGobalConfig();
			data.writeProjectInfo(true, false);
			
			dispose();
			
		}
	};

	/**
	 * Closes the dialog without doing anything
	 */
	ActionListener closeDialog = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			dispose();
		}
	};

	/**
	 * Allow the user to pick a plink file from a file chooser
	 */
	ActionListener pickPlink = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			if(frame.data.isRemote(true) == true){
				new RemoteFileChooser(ConfigDialog.this, frame.data.getConn(), 
						plinkP, frame.data.getRemoteFile(), 
						false);
			} else {
				// create a new dialog to pick the directory
				JFileChooser fc = new JFileChooser();
				// filter the files so that we only see .ped files
				fc.addChoosableFileFilter(new PlinkFilter());
				// only look at directories and files
				fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				// set the directory in the file chooser to default to the
				// ...root file of the project
				fc.setCurrentDirectory(frame.data.getRoot());
				// show the dialog and when it is OK-ed
				if (fc.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
					// if a directory is selected then get out without setting
					// ..anything
					if (fc.getSelectedFile().isDirectory())
						return;
					else
						// otherwise we have a valid file and set it to the
						// ...text box
						plinkP.setText(fc.getSelectedFile().getAbsolutePath());
				}
			}
		}

	};
	
	/**
	 * Allow the user to pick a plink file from a file chooser
	 */
	ActionListener pickHaploview = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			// create a new dialog to pick the directory
			JFileChooser fc = new JFileChooser();
			// filter the files so that we only see .ped files
			fc.addChoosableFileFilter(new JavaFilter());
			// only look at directories and files
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			// set the directory in the file chooser to default to the
			// ...root file of the project
			fc.setCurrentDirectory(frame.data.getRoot());
			// show the dialog and when it is OK-ed
			if (fc.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
				// if a directory is selected then get out without setting
				// ..anything
				if (fc.getSelectedFile().isDirectory())
					return;
				else
					// otherwise we have a valid file and set it to the
					// ...text box
					haploviewPath.setText(fc.getSelectedFile().getAbsolutePath());
			}
		}

	};

	/**
	 * Allow the user to pick a plink file from a file chooser
	 */
	ActionListener pickEditor = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			// create a new dialog to pick the directory
			JFileChooser fc = new JFileChooser();
			// only look at directories and files
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			// set the directory in the file chooser to default to the
			// ...root file of the project
			fc.setCurrentDirectory(frame.data.getRoot());
			// show the dialog and when it is OK-ed
			if (fc.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
				userEditorName.setText(fc.getSelectedFile().getAbsolutePath());
			}
		}

	};
	
	ActionListener testEditor = new ActionListener(){
		public void actionPerformed(ActionEvent e){
			try {
				@SuppressWarnings("unused") 
				//Process testing = Runtime.getRuntime().exec(userEditorName.getText());
				Process testing = Runtime.getRuntime().exec(
						GeneralUtil.stripAndSplit(userEditorName.getText()));
				//Process testing = Runtime.getRuntime().exec(new String[]{
				//		"emacs /home/ktoddbrown/testing spaces/alk.log"});
			} catch (IOException e1) {
				new ErrorDialog(frame,
        			"Your test did not work, " +
        			"can not execute this command.\n" +
        			"[ " + userEditorName.getText() + " ]" );
			}
		}
	};
		
	/**
	 * create a panel to hold the fields needed for picking the plink path
	 * @return the panel that asks for the plink path
	 */
	private JPanel makePlinkPane() {
		//create a new panel
		JPanel ans = new JPanel();
		//label what we are looking at
		JLabel plinkL = new JLabel("PLINK path:");
		//give the user a place to type
		plinkP = new JTextField(data.getPlinkPath(), 40);
		//create a button that will let the user select a file
		JButton browse = new JButton("Browse");
		//tell the button what to do when it's clicked
		browse.addActionListener(pickPlink);
		//add things to the panel
		ans.add(plinkL);
		ans.add(plinkP);
		ans.add(browse);
		
		return ans;
	}

	/**
	 * create a panel to hold the fields needed for picking the editor
	 * @return the panel that asks for what editor to use
	 */
	private JPanel makeEditorPane(){
		//create the panel we will return
		JPanel ans = new JPanel();
		//set the layout for that panel
		ans.setLayout(new BoxLayout(ans, BoxLayout.PAGE_AXIS));
		//tell the user what panel we are looking at
		ans.setBorder(new TitledBorder("Editor options"));
		//button to select the default editor
		defaultEditor = new JLabel("Default editor: " 
				+ data.getDefaultEditor());
		//button to select the user defied editor
		userEditor = new JCheckBox("User specified editor: ");
		//populate the user defined editor
		userEditorName = new JTextField(data.getEditor(), 20);
		//button to pop up a file chooser
		JButton browse = new JButton("Browse");
		browse.addActionListener(pickEditor);
		userEditor.setSelected(!data.getEditor().equals(""));
		JButton test = new JButton("Test");
		test.addActionListener(testEditor);
		
		//add things to the first row using a seperate panel
		JPanel userPanel = new JPanel();
		userPanel.add(userEditor);
		userPanel.add(userEditorName);
		userPanel.add(browse);
		userPanel.add(test);
		
		//add everything to the main panel
		ans.add(defaultEditor);
		ans.add(userPanel);
		return ans;
	}
	
	/**
	 * create a panel to hold the fields needed for setting the HaploView path
	 * @return the panel that asks for the HaploView path
	 */
	private JPanel makeHaploPane(){

		JPanel ans = new JPanel();
		ans.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		JLabel hapL = new JLabel("Haploview .jar path:");
		JLabel hapA = new JLabel("Haploview optional append");
		haploviewPath = new JTextField(
				data.getHaploviewPath(), 40);
		haploviewAppend = new JTextField(data.getHaploviewAppend());
		JButton browse = new JButton("Browse");
		browse.addActionListener(pickHaploview);

		ans.add(hapL, c);
		c.gridx = 1;
		c.weightx = 1;
		ans.add(haploviewPath, c);
		c.gridx = 2;
		c.weightx = 0;
		ans.add(browse, c);
		
		c.gridy = 1;
		c.gridx = 0;
		ans.add(hapA, c);
		c.gridx = 1;
		c.weightx = 1;
		c.gridwidth = 2;
		ans.add(haploviewAppend, c);

		return ans;
	}
	
	/**
	 * create a panel for the ok and cancel buttons
	 * @return a panel with the ok and cancel buttons in it
	 */
	private JPanel makeButtonPane() {
		JPanel ans = new JPanel();

		JButton okButton = new JButton("OK");
		JButton cancelButton = new JButton("Cancel");

		okButton.addActionListener(setConfigValues);
		cancelButton.addActionListener(closeDialog);

		ans.add(okButton);
		ans.add(cancelButton);

		return ans;
	}

	/**
	 * Create a panel with the plink prefix fields in it
	 * @return a panel for the plink prefix field
	 */
	private JPanel makePrefixPane() {
		JPanel ans = new JPanel();

		JLabel info = new JLabel("PLINK prefix: ");
		prefix = new JTextField(data.getPlinkPrefix(), 30);

		ans.add(info);
		ans.add(prefix);

		return ans;

	}
	
	/**
	 * construct a configurations dialog
	 * @param mf
	 */
	public ConfigDialog(MainFrame mf) {
		//attach the dialog to mf
		super(mf, "Project Configuration");
		//set the frame
		frame = mf;
		//set the project
		data = mf.data;
		config = new Properties();
		
		//set the layout
		getContentPane().setLayout(
				new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridy = 0;
		//add all the panels
		getContentPane().add(makePrefixPane(), c);
		c.gridy ++;
		getContentPane().add(makePlinkPane(), c);
		c.gridy ++;
		getContentPane().add(makeEditorPane(), c);
		c.gridy ++;
		getContentPane().add(makeHaploPane(), c);
		c.gridy ++;
		getContentPane().add(makeButtonPane(), c);

		//pack it up
		pack();
		
		//set it as visible
		setVisible(true);
	}

	/**
	 * Filter for plink
	 * 
	 * @param f1
	 */
	public static class PlinkFilter extends FileFilter {
		public boolean accept(File f) {
			return (f.getName().contains("plink") || f.isDirectory());
		}

		public String getDescription() {
			return "PLINK files";
		}
	}
	
	/**
	 * Filter for java .jar
	 * 
	 * @param f1
	 */
	public static class JavaFilter extends FileFilter {
		public boolean accept(File f) {
			return (f.getName().endsWith(".jar") || f.isDirectory());
		}

		public String getDescription() {
			return "JAR";
		}
	}
}
